@@ -29,6 +29,7 @@
 #include "config.h"
 #endif
 
+#include "cogl-util.h"
 #include "cogl-context-private.h"
 #include "cogl-object-private.h"
 #include "cogl-journal-private.h"
@@ -47,6 +48,7 @@
 
 #include <string.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 /* This isn't defined in the GLES headers */
 #ifndef GL_UNSIGNED_INT
@@ -163,7 +165,9 @@ validate_cogl_attribute (const char *name,
     *name_id = COGL_ATTRIBUTE_NAME_ID_TEXTURE_COORD_ARRAY;
   else if (strncmp (name, "tex_coord", strlen ("tex_coord")) == 0)
     {
-      if (sscanf (name, "tex_coord%u_in", texture_unit) != 1)
+      char *endptr;
+      *texture_unit = strtoul (name + 9, &endptr, 10);
+      if (strcmp (endptr, "_in") != 0)
 	{
 	  g_warning ("Texture coordinate attributes should either be named "
                      "\"cogl_tex_coord\" or named with a texture unit index "
@@ -205,7 +209,6 @@ cogl_attribute_new (CoglAttributeBuffer *attribute_buffer,
   gboolean status;
 
   attribute->attribute_buffer = cogl_object_ref (attribute_buffer);
-  attribute->name = g_strdup (name);
   attribute->stride = stride;
   attribute->offset = offset;
   attribute->n_components = n_components;
@@ -213,11 +216,46 @@ cogl_attribute_new (CoglAttributeBuffer *attribute_buffer,
   attribute->immutable_ref = 0;
 
   if (strncmp (name, "cogl_", 5) == 0)
-    status = validate_cogl_attribute (attribute->name,
-                                      n_components,
-                                      &attribute->name_id,
-                                      &attribute->normalized,
-                                      &attribute->texture_unit);
+    {
+      const char *common_tex_coord_names[8] = {
+          "cogl_tex_coord0_in",
+          "cogl_tex_coord1_in",
+          "cogl_tex_coord2_in",
+          "cogl_tex_coord3_in",
+          "cogl_tex_coord4_in",
+          "cogl_tex_coord5_in",
+          "cogl_tex_coord6_in",
+          "cogl_tex_coord7_in"
+      };
+      status = validate_cogl_attribute (name,
+                                        n_components,
+                                        &attribute->name_id,
+                                        &attribute->normalized,
+                                        &attribute->texture_unit);
+
+      /* Avoid even the cost of g_intern_string() for the very common case
+       * attribute names...*/
+      switch (attribute->name_id)
+        {
+        case COGL_ATTRIBUTE_NAME_ID_POSITION_ARRAY:
+          attribute->name = "cogl_position_in";
+          break;
+        case COGL_ATTRIBUTE_NAME_ID_COLOR_ARRAY:
+          attribute->name = "cogl_color_in";
+          break;
+        case COGL_ATTRIBUTE_NAME_ID_TEXTURE_COORD_ARRAY:
+          if (attribute->texture_unit < 8)
+            attribute->name = common_tex_coord_names[attribute->texture_unit];
+          else
+            attribute->name = g_intern_string (name);
+          break;
+        case COGL_ATTRIBUTE_NAME_ID_NORMAL_ARRAY:
+          attribute->name = "cogl_normal_in";
+          break;
+        default:
+          g_warn_if_reached ();
+        }
+    }
 #if 0
   else if (strncmp (name, "gl_", 3) == 0)
     status = validate_gl_attribute (attribute->name,
@@ -228,6 +266,7 @@ cogl_attribute_new (CoglAttributeBuffer *attribute_buffer,
 #endif
   else
     {
+      attribute->name = g_intern_string (name);
       attribute->name_id = COGL_ATTRIBUTE_NAME_ID_CUSTOM_ARRAY;
       attribute->normalized = FALSE;
       attribute->texture_unit = 0;
@@ -246,7 +285,7 @@ cogl_attribute_new (CoglAttributeBuffer *attribute_buffer,
 gboolean
 cogl_attribute_get_normalized (CoglAttribute *attribute)
 {
-  g_return_val_if_fail (cogl_is_attribute (attribute), FALSE);
+  _COGL_RETURN_VAL_IF_FAIL (cogl_is_attribute (attribute), FALSE);
 
   return attribute->normalized;
 }
@@ -267,7 +306,7 @@ void
 cogl_attribute_set_normalized (CoglAttribute *attribute,
                                       gboolean normalized)
 {
-  g_return_if_fail (cogl_is_attribute (attribute));
+  _COGL_RETURN_IF_FAIL (cogl_is_attribute (attribute));
 
   if (G_UNLIKELY (attribute->immutable_ref))
     warn_about_midscene_changes ();
@@ -278,7 +317,7 @@ cogl_attribute_set_normalized (CoglAttribute *attribute,
 CoglAttributeBuffer *
 cogl_attribute_get_buffer (CoglAttribute *attribute)
 {
-  g_return_val_if_fail (cogl_is_attribute (attribute), NULL);
+  _COGL_RETURN_VAL_IF_FAIL (cogl_is_attribute (attribute), NULL);
 
   return attribute->attribute_buffer;
 }
@@ -287,7 +326,7 @@ void
 cogl_attribute_set_buffer (CoglAttribute *attribute,
                            CoglAttributeBuffer *attribute_buffer)
 {
-  g_return_if_fail (cogl_is_attribute (attribute));
+  _COGL_RETURN_IF_FAIL (cogl_is_attribute (attribute));
 
   if (G_UNLIKELY (attribute->immutable_ref))
     warn_about_midscene_changes ();
@@ -301,7 +340,7 @@ cogl_attribute_set_buffer (CoglAttribute *attribute,
 CoglAttribute *
 _cogl_attribute_immutable_ref (CoglAttribute *attribute)
 {
-  g_return_val_if_fail (cogl_is_attribute (attribute), NULL);
+  _COGL_RETURN_VAL_IF_FAIL (cogl_is_attribute (attribute), NULL);
 
   attribute->immutable_ref++;
   _cogl_buffer_immutable_ref (COGL_BUFFER (attribute->attribute_buffer));
@@ -311,8 +350,8 @@ _cogl_attribute_immutable_ref (CoglAttribute *attribute)
 void
 _cogl_attribute_immutable_unref (CoglAttribute *attribute)
 {
-  g_return_if_fail (cogl_is_attribute (attribute));
-  g_return_if_fail (attribute->immutable_ref > 0);
+  _COGL_RETURN_IF_FAIL (cogl_is_attribute (attribute));
+  _COGL_RETURN_IF_FAIL (attribute->immutable_ref > 0);
 
   attribute->immutable_ref--;
   _cogl_buffer_immutable_unref (COGL_BUFFER (attribute->attribute_buffer));
@@ -321,7 +360,6 @@ _cogl_attribute_immutable_unref (CoglAttribute *attribute)
 static void
 _cogl_attribute_free (CoglAttribute *attribute)
 {
-  g_free (attribute->name);
   cogl_object_unref (attribute->attribute_buffer);
 
   g_slice_free (CoglAttribute, attribute);
@@ -339,14 +377,14 @@ validate_layer_cb (CoglPipeline *pipeline,
                    int layer_index,
                    void *user_data)
 {
-  CoglHandle texture =
+  CoglTexture *texture =
     _cogl_pipeline_get_layer_texture (pipeline, layer_index);
   ValidateLayerState *state = user_data;
   gboolean status = TRUE;
 
   /* invalid textures will be handled correctly in
    * _cogl_pipeline_flush_layers_gl_state */
-  if (texture == COGL_INVALID_HANDLE)
+  if (texture == NULL)
     goto validated;
 
   _cogl_texture_flush_journal_rendering (texture);
@@ -390,13 +428,13 @@ validated:
   return status;
 }
 
-static void
+static gboolean
 toggle_enabled_cb (int bit_num, void *user_data)
 {
   const CoglBitmask *new_values = user_data;
   gboolean enabled = _cogl_bitmask_get (new_values, bit_num);
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+  _COGL_GET_CONTEXT (ctx, FALSE);
 
   if (ctx->driver == COGL_DRIVER_GLES2)
     {
@@ -416,6 +454,8 @@ toggle_enabled_cb (int bit_num, void *user_data)
         GE( ctx, glDisableClientState (GL_TEXTURE_COORD_ARRAY) );
     }
 #endif
+
+  return TRUE;
 }
 
 static void
