@@ -30,7 +30,9 @@
 #endif
 
 #include "cogl-debug.h"
+#include "cogl-context-private.h"
 #include "cogl-pipeline-private.h"
+#include "cogl-pipeline-layer-private.h"
 
 #ifdef COGL_PIPELINE_FRAGEND_ARBFP
 
@@ -61,6 +63,7 @@ typedef struct _UnitState
 {
   int constant_id; /* The program.local[] index */
   unsigned int dirty_combine_constant:1;
+  unsigned int has_combine_constant:1;
 
   unsigned int sampled:1;
 } UnitState;
@@ -106,12 +109,20 @@ get_shader_state (CoglPipeline *pipeline)
 }
 
 static void
-destroy_shader_state (void *user_data)
+destroy_shader_state (void *user_data,
+                      void *instance)
 {
   CoglPipelineShaderState *shader_state = user_data;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  /* If the shader state was last used for this pipeline then clear it
+     so that if same address gets used again for a new pipeline then
+     we won't think it's the same pipeline and avoid updating the
+     constants */
+  if (shader_state->last_used_for_pipeline == instance)
+    shader_state->last_used_for_pipeline = NULL;
+
   if (--shader_state->ref_count == 0)
     {
       if (shader_state->gl_program)
@@ -129,10 +140,10 @@ destroy_shader_state (void *user_data)
 static void
 set_shader_state (CoglPipeline *pipeline, CoglPipelineShaderState *shader_state)
 {
-  cogl_object_set_user_data (COGL_OBJECT (pipeline),
-                             &shader_state_key,
-                             shader_state,
-                             destroy_shader_state);
+  _cogl_object_set_user_data (COGL_OBJECT (pipeline),
+                              &shader_state_key,
+                              shader_state,
+                              destroy_shader_state);
 }
 
 static void
@@ -160,11 +171,11 @@ _cogl_pipeline_fragend_arbfp_start (CoglPipeline *pipeline,
   /* First validate that we can handle the current state using ARBfp
    */
 
-  if (!cogl_features_available (COGL_FEATURE_SHADERS_ARBFP))
+  if (!cogl_has_feature (ctx, COGL_FEATURE_ID_ARBFP))
     return FALSE;
 
   /* TODO: support fog */
-  if (ctx->legacy_fog_state.enabled)
+  if (_cogl_pipeline_get_fog_enabled (pipeline))
     return FALSE;
 
   user_program = cogl_pipeline_get_user_program (pipeline);
@@ -238,8 +249,6 @@ _cogl_pipeline_fragend_arbfp_start (CoglPipeline *pipeline,
       shader_state->user_program = user_program;
       if (user_program == COGL_INVALID_HANDLE)
         {
-          int i;
-
           /* We reuse a single grow-only GString for code-gen */
           g_string_set_size (ctx->codegen_source_buffer, 0);
           shader_state->source = ctx->codegen_source_buffer;
@@ -251,13 +260,6 @@ _cogl_pipeline_fragend_arbfp_start (CoglPipeline *pipeline,
                            "PARAM one = {1, 1, 1, 1};\n"
                            "PARAM two = {2, 2, 2, 2};\n"
                            "PARAM minus_one = {-1, -1, -1, -1};\n");
-
-          for (i = 0; i < n_layers; i++)
-            {
-              shader_state->unit_state[i].sampled = FALSE;
-              shader_state->unit_state[i].dirty_combine_constant = FALSE;
-            }
-          shader_state->next_constant_id = 0;
         }
     }
 
@@ -395,7 +397,10 @@ setup_arg (CoglPipeline *pipeline,
       arg->name = "texel%d";
       arg->texture_unit = _cogl_pipeline_layer_get_unit_index (layer);
       texture = _cogl_pipeline_layer_get_texture (layer);
-      cogl_texture_get_gl_texture (texture, NULL, &gl_target);
+      if (texture)
+        cogl_texture_get_gl_texture (texture, NULL, &gl_target);
+      else
+        gl_target = GL_TEXTURE_2D;
       setup_texture_source (shader_state, arg->texture_unit, gl_target);
       break;
     case COGL_PIPELINE_COMBINE_SOURCE_CONSTANT:
@@ -404,6 +409,7 @@ setup_arg (CoglPipeline *pipeline,
         UnitState *unit_state = &shader_state->unit_state[unit_index];
 
         unit_state->constant_id = shader_state->next_constant_id++;
+        unit_state->has_combine_constant = TRUE;
         unit_state->dirty_combine_constant = TRUE;
 
         arg->type = COGL_PIPELINE_FRAGEND_ARBFP_ARG_TYPE_CONSTANT;
@@ -427,7 +433,10 @@ setup_arg (CoglPipeline *pipeline,
       arg->name = "texture[%d]";
       arg->texture_unit = src - GL_TEXTURE0;
       texture = _cogl_pipeline_layer_get_texture (layer);
-      cogl_texture_get_gl_texture (texture, NULL, &gl_target);
+      if (texture)
+        cogl_texture_get_gl_texture (texture, NULL, &gl_target);
+      else
+        gl_target = GL_TEXTURE_2D;
       setup_texture_source (shader_state, arg->texture_unit, gl_target);
     }
 
@@ -714,7 +723,7 @@ _cogl_pipeline_fragend_arbfp_add_layer (CoglPipeline *pipeline,
   if (!shader_state->source)
     return TRUE;
 
-  if (!_cogl_pipeline_need_texture_combine_separate (combine_authority))
+  if (!_cogl_pipeline_layer_needs_combine_separate (combine_authority))
     {
       append_masked_combine (pipeline,
                              layer,
@@ -786,7 +795,8 @@ update_constants_cb (CoglPipeline *pipeline,
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
-  if (state->update_all || unit_state->dirty_combine_constant)
+  if (unit_state->has_combine_constant &&
+      (state->update_all || unit_state->dirty_combine_constant))
     {
       float constant[4];
       _cogl_pipeline_get_layer_combine_constant (pipeline,
