@@ -36,6 +36,8 @@
 #include "cogl-vertex-buffer-private.h"
 #include "cogl-framebuffer-private.h"
 #include "cogl-attribute-private.h"
+#include "cogl-private.h"
+#include "cogl-meta-texture.h"
 
 #include <string.h>
 #include <math.h>
@@ -45,7 +47,7 @@
 typedef struct _TextureSlicedQuadState
 {
   CoglPipeline *pipeline;
-  CoglHandle main_texture;
+  CoglTexture *main_texture;
   float tex_virtual_origin_x;
   float tex_virtual_origin_y;
   float quad_origin_x;
@@ -67,14 +69,14 @@ typedef struct _TextureSlicedPolygonState
 } TextureSlicedPolygonState;
 
 static void
-log_quad_sub_textures_cb (CoglHandle texture_handle,
+log_quad_sub_textures_cb (CoglTexture *texture,
                           const float *subtexture_coords,
                           const float *virtual_coords,
                           void *user_data)
 {
   TextureSlicedQuadState *state = user_data;
   CoglFramebuffer *framebuffer = cogl_get_draw_framebuffer ();
-  CoglHandle texture_override;
+  CoglTexture *texture_override;
   float quad_coords[4];
 
 #define TEX_VIRTUAL_TO_QUAD(V, Q, AXIS) \
@@ -111,10 +113,10 @@ log_quad_sub_textures_cb (CoglHandle texture_handle,
 
   /* We only need to override the texture if it's different from the
      main texture */
-  if (texture_handle == state->main_texture)
-    texture_override = COGL_INVALID_HANDLE;
+  if (texture == state->main_texture)
+    texture_override = NULL;
   else
-    texture_override = texture_handle;
+    texture_override = texture;
 
   _cogl_journal_log_quad (framebuffer->journal,
                           quad_coords,
@@ -138,27 +140,33 @@ validate_first_layer_cb (CoglPipeline *pipeline,
   ValidateFirstLayerState *state = user_data;
   CoglPipelineWrapMode clamp_to_edge =
     COGL_PIPELINE_WRAP_MODE_CLAMP_TO_EDGE;
+  CoglPipelineWrapMode wrap_s;
+  CoglPipelineWrapMode wrap_t;
 
   /* We can't use hardware repeat so we need to set clamp to edge
    * otherwise it might pull in edge pixels from the other side. By
    * default WRAP_MODE_AUTOMATIC becomes CLAMP_TO_EDGE so we only need
-   * to override if the wrap mode is repeat.
+   * to override if the wrap mode isn't already automatic or
+   * clamp_to_edge.
    */
-  if (cogl_pipeline_get_layer_wrap_mode_s (pipeline, layer_index) ==
-      COGL_PIPELINE_WRAP_MODE_REPEAT)
+  wrap_s = cogl_pipeline_get_layer_wrap_mode_s (pipeline, layer_index);
+  if (wrap_s != COGL_PIPELINE_WRAP_MODE_CLAMP_TO_EDGE &&
+      wrap_s != COGL_PIPELINE_WRAP_MODE_AUTOMATIC)
     {
       if (!state->override_pipeline)
         state->override_pipeline = cogl_pipeline_copy (pipeline);
-      cogl_pipeline_set_layer_wrap_mode_s (pipeline, layer_index,
-                                           clamp_to_edge);
+      cogl_pipeline_set_layer_wrap_mode_s (state->override_pipeline,
+                                           layer_index, clamp_to_edge);
     }
-  if (cogl_pipeline_get_layer_wrap_mode_t (pipeline, layer_index) ==
-      COGL_PIPELINE_WRAP_MODE_REPEAT)
+
+  wrap_t = cogl_pipeline_get_layer_wrap_mode_t (pipeline, layer_index);
+  if (wrap_t != COGL_PIPELINE_WRAP_MODE_CLAMP_TO_EDGE &&
+      wrap_t != COGL_PIPELINE_WRAP_MODE_AUTOMATIC)
     {
       if (!state->override_pipeline)
         state->override_pipeline = cogl_pipeline_copy (pipeline);
-      cogl_pipeline_set_layer_wrap_mode_t (pipeline, layer_index,
-                                           clamp_to_edge);
+      cogl_pipeline_set_layer_wrap_mode_t (state->override_pipeline,
+                                           layer_index, clamp_to_edge);
     }
 
   return FALSE;
@@ -179,15 +187,14 @@ validate_first_layer_cb (CoglPipeline *pipeline,
  */
 /* TODO: support multitexturing */
 static void
-_cogl_texture_quad_multiple_primitives (CoglHandle    tex_handle,
+_cogl_texture_quad_multiple_primitives (CoglTexture *texture,
                                         CoglPipeline *pipeline,
-                                        gboolean      clamp_s,
-                                        gboolean      clamp_t,
-                                        const float  *position,
-                                        float         tx_1,
-                                        float         ty_1,
-                                        float         tx_2,
-                                        float         ty_2)
+                                        int layer_index,
+                                        const float *position,
+                                        float tx_1,
+                                        float ty_1,
+                                        float tx_2,
+                                        float ty_2)
 {
   TextureSlicedQuadState state;
   gboolean tex_virtual_flipped_x;
@@ -195,113 +202,19 @@ _cogl_texture_quad_multiple_primitives (CoglHandle    tex_handle,
   gboolean quad_flipped_x;
   gboolean quad_flipped_y;
   ValidateFirstLayerState validate_first_layer_state;
+  CoglPipelineWrapMode wrap_s, wrap_t;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  /* If the wrap mode is clamp to edge then we'll recursively draw the
-     stretched part and replace the coordinates */
-  if (clamp_s && tx_1 != tx_2)
-    {
-      float *replacement_position = g_newa (float, 4);
-      float old_tx_1 = tx_1, old_tx_2 = tx_2;
-
-      memcpy (replacement_position, position, sizeof (float) * 4);
-
-      tx_1 = CLAMP (tx_1, 0.0f, 1.0f);
-      tx_2 = CLAMP (tx_2, 0.0f, 1.0f);
-
-      if (old_tx_1 != tx_1)
-        {
-          /* Draw the left part of the quad as a stretched copy of tx_1 */
-          float tmp_position[] =
-            { position[0], position[1],
-              (position[0] +
-               (position[2] - position[0]) *
-               (tx_1 - old_tx_1) / (old_tx_2 - old_tx_1)),
-              position[3] };
-          _cogl_texture_quad_multiple_primitives (tex_handle, pipeline,
-                                                  FALSE, clamp_t,
-                                                  tmp_position,
-                                                  tx_1, ty_1, tx_1, ty_2);
-          replacement_position[0] = tmp_position[2];
-        }
-
-      if (old_tx_2 != tx_2)
-        {
-          /* Draw the right part of the quad as a stretched copy of tx_2 */
-          float tmp_position[] =
-            { (position[0] +
-               (position[2] - position[0]) *
-               (tx_2 - old_tx_1) / (old_tx_2 - old_tx_1)),
-              position[1], position[2], position[3] };
-          _cogl_texture_quad_multiple_primitives (tex_handle, pipeline,
-                                                  FALSE, clamp_t,
-                                                  tmp_position,
-                                                  tx_2, ty_1, tx_2, ty_2);
-          replacement_position[2] = tmp_position[0];
-        }
-
-      /* If there's no main part left then we don't need to continue */
-      if (tx_1 == tx_2)
-        return;
-
-      position = replacement_position;
-    }
-
-  if (clamp_t && ty_1 != ty_2)
-    {
-      float *replacement_position = g_newa (float, 4);
-      float old_ty_1 = ty_1, old_ty_2 = ty_2;
-
-      memcpy (replacement_position, position, sizeof (float) * 4);
-
-      ty_1 = CLAMP (ty_1, 0.0f, 1.0f);
-      ty_2 = CLAMP (ty_2, 0.0f, 1.0f);
-
-      if (old_ty_1 != ty_1)
-        {
-          /* Draw the top part of the quad as a stretched copy of ty_1 */
-          float tmp_position[] =
-            { position[0], position[1], position[2],
-              (position[1] +
-               (position[3] - position[1]) *
-               (ty_1 - old_ty_1) / (old_ty_2 - old_ty_1)) };
-          _cogl_texture_quad_multiple_primitives (tex_handle, pipeline,
-                                                  clamp_s, FALSE,
-                                                  tmp_position,
-                                                  tx_1, ty_1, tx_2, ty_1);
-          replacement_position[1] = tmp_position[3];
-        }
-
-      if (old_ty_2 != ty_2)
-        {
-          /* Draw the bottom part of the quad as a stretched copy of ty_2 */
-          float tmp_position[] =
-            { position[0],
-              (position[1] +
-               (position[3] - position[1]) *
-               (ty_2 - old_ty_1) / (old_ty_2 - old_ty_1)),
-              position[2], position[3] };
-          _cogl_texture_quad_multiple_primitives (tex_handle, pipeline,
-                                                  clamp_s, FALSE,
-                                                  tmp_position,
-                                                  tx_1, ty_2, tx_2, ty_2);
-          replacement_position[3] = tmp_position[1];
-        }
-
-      /* If there's no main part left then we don't need to continue */
-      if (ty_1 == ty_2)
-        return;
-
-      position = replacement_position;
-    }
+  wrap_s = cogl_pipeline_get_layer_wrap_mode_s (pipeline, layer_index);
+  wrap_t = cogl_pipeline_get_layer_wrap_mode_t (pipeline, layer_index);
 
   validate_first_layer_state.override_pipeline = NULL;
   cogl_pipeline_foreach_layer (pipeline,
                                validate_first_layer_cb,
                                &validate_first_layer_state);
 
-  state.main_texture = tex_handle;
+  state.main_texture = texture;
 
   if (validate_first_layer_state.override_pipeline)
     state.pipeline = validate_first_layer_state.override_pipeline;
@@ -349,10 +262,19 @@ _cogl_texture_quad_multiple_primitives (CoglHandle    tex_handle,
   state.v_to_q_scale_x = fabs (state.quad_len_x / (tx_2 - tx_1));
   state.v_to_q_scale_y = fabs (state.quad_len_y / (ty_2 - ty_1));
 
-  _cogl_texture_foreach_sub_texture_in_region (tex_handle,
-                                               tx_1, ty_1, tx_2, ty_2,
-                                               log_quad_sub_textures_cb,
-                                               &state);
+  /* For backwards compatablity the default wrap mode for cogl_rectangle() is
+   * _REPEAT... */
+  if (wrap_s == COGL_PIPELINE_WRAP_MODE_AUTOMATIC)
+    wrap_s = COGL_PIPELINE_WRAP_MODE_REPEAT;
+  if (wrap_t == COGL_PIPELINE_WRAP_MODE_AUTOMATIC)
+    wrap_t = COGL_PIPELINE_WRAP_MODE_REPEAT;
+
+  cogl_meta_texture_foreach_in_region (COGL_META_TEXTURE (texture),
+                                       tx_1, ty_1, tx_2, ty_2,
+                                       wrap_s,
+                                       wrap_t,
+                                       log_quad_sub_textures_cb,
+                                       &state);
 
   if (validate_first_layer_state.override_pipeline)
     cogl_object_unref (validate_first_layer_state.override_pipeline);
@@ -378,7 +300,7 @@ validate_tex_coords_cb (CoglPipeline *pipeline,
                         void *user_data)
 {
   ValidateTexCoordsState *state = user_data;
-  CoglHandle texture;
+  CoglTexture *texture;
   const float *in_tex_coords;
   float *out_tex_coords;
   float default_tex_coords[4] = {0.0, 0.0, 1.0, 1.0};
@@ -456,7 +378,7 @@ validate_tex_coords_cb (CoglPipeline *pipeline,
                        "supported with multi-texturing.", state->i);
           warning_seen = TRUE;
 
-          cogl_pipeline_set_layer_texture (texture, layer_index, NULL);
+          cogl_pipeline_set_layer_texture (pipeline, layer_index, NULL);
         }
     }
 
@@ -542,7 +464,7 @@ _cogl_multitexture_quad_single_primitive (const float  *position,
                           position,
                           pipeline,
                           n_layers,
-                          COGL_INVALID_HANDLE, /* no texture override */
+                          NULL, /* no texture override */
                           final_tex_coords,
                           n_layers * 4);
 
@@ -566,7 +488,7 @@ _cogl_rectangles_validate_layer_cb (CoglPipeline *pipeline,
                                     void *user_data)
 {
   ValidateLayerState *state = user_data;
-  CoglHandle texture;
+  CoglTexture *texture;
 
   state->i++;
 
@@ -614,9 +536,9 @@ _cogl_rectangles_validate_layer_cb (CoglPipeline *pipeline,
 
   texture = _cogl_pipeline_get_layer_texture (pipeline, layer_index);
 
-  /* COGL_INVALID_HANDLE textures are handled by
+  /* NULL textures are handled by
    * _cogl_pipeline_flush_gl_state */
-  if (texture == COGL_INVALID_HANDLE)
+  if (texture == NULL)
     return TRUE;
 
   if (state->i == 0)
@@ -635,21 +557,25 @@ _cogl_rectangles_validate_layer_cb (CoglPipeline *pipeline,
     {
       if (state->i == 0)
         {
-          static gboolean warning_seen = FALSE;
+          if (cogl_pipeline_get_n_layers (pipeline) > 1)
+            {
+              static gboolean warning_seen = FALSE;
+
+              if (!state->override_source)
+                state->override_source = cogl_pipeline_copy (pipeline);
+              _cogl_pipeline_prune_to_n_layers (state->override_source, 1);
+
+              if (!warning_seen)
+                g_warning ("Skipping layers 1..n of your pipeline since "
+                           "the first layer is sliced. We don't currently "
+                           "support any multi-texturing with sliced "
+                           "textures but assume layer 0 is the most "
+                           "important to keep");
+              warning_seen = TRUE;
+            }
 
-          if (!state->override_source)
-            state->override_source = cogl_pipeline_copy (pipeline);
-          _cogl_pipeline_prune_to_n_layers (state->override_source, 1);
           state->all_use_sliced_quad_fallback = TRUE;
 
-          if (!warning_seen)
-            g_warning ("Skipping layers 1..n of your pipeline since "
-                       "the first layer is sliced. We don't currently "
-                       "support any multi-texturing with sliced "
-                       "textures but assume layer 0 is the most "
-                       "important to keep");
-          warning_seen = TRUE;
-
           return FALSE;
         }
       else
@@ -711,13 +637,13 @@ _cogl_rectangles_with_multitexture_coords (
                                         struct _CoglMutiTexturedRect *rects,
                                         int                           n_rects)
 {
-  CoglPipeline *pipeline;
+  CoglPipeline *original_pipeline, *pipeline;
   ValidateLayerState state;
   int i;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  pipeline = cogl_get_source ();
+  pipeline = original_pipeline = cogl_get_source ();
 
   /*
    * Validate all the layers of the current source pipeline...
@@ -733,16 +659,24 @@ _cogl_rectangles_with_multitexture_coords (
   if (state.override_source)
     pipeline = state.override_source;
 
+  if (G_UNLIKELY (ctx->legacy_state_set) &&
+      _cogl_get_enable_legacy_state ())
+    {
+      /* If we haven't already made a pipeline copy */
+      if (pipeline == original_pipeline)
+        pipeline = cogl_pipeline_copy (pipeline);
+      _cogl_pipeline_apply_legacy_state (pipeline);
+    }
+
   /*
    * Emit geometry for each of the rectangles...
    */
 
   for (i = 0; i < n_rects; i++)
     {
-      CoglHandle texture;
+      CoglTexture *texture;
       const float default_tex_coords[4] = {0.0, 0.0, 1.0, 1.0};
       const float *tex_coords;
-      gboolean clamp_s, clamp_t;
 
       if (!state.all_use_sliced_quad_fallback)
         {
@@ -770,18 +704,11 @@ _cogl_rectangles_with_multitexture_coords (
       else
         tex_coords = default_tex_coords;
 
-      clamp_s = (cogl_pipeline_get_layer_wrap_mode_s (pipeline,
-                                                      state.first_layer) ==
-                 COGL_PIPELINE_WRAP_MODE_CLAMP_TO_EDGE);
-      clamp_t = (cogl_pipeline_get_layer_wrap_mode_t (pipeline,
-                                                      state.first_layer) ==
-                 COGL_PIPELINE_WRAP_MODE_CLAMP_TO_EDGE);
-
       COGL_NOTE (DRAW, "Drawing Tex Quad (Multi-Prim Mode)");
 
       _cogl_texture_quad_multiple_primitives (texture,
                                               pipeline,
-                                              clamp_s, clamp_t,
+                                              state.first_layer,
                                               rects[i].position,
                                               tex_coords[0],
                                               tex_coords[1],
@@ -789,7 +716,7 @@ _cogl_rectangles_with_multitexture_coords (
                                               tex_coords[3]);
     }
 
-  if (state.override_source)
+  if (pipeline != original_pipeline)
     cogl_object_unref (pipeline);
 }
 
@@ -967,19 +894,19 @@ append_tex_coord_attributes_cb (CoglPipeline *pipeline,
                                 void *user_data)
 {
   AppendTexCoordsState *state = user_data;
-  CoglHandle tex_handle;
+  CoglTexture *texture;
   float tx, ty;
   float *t;
 
   tx = state->vertices_in[state->vertex].tx;
   ty = state->vertices_in[state->vertex].ty;
 
-  /* COGL_INVALID_HANDLE textures will be handled in
+  /* NULL textures will be handled in
    * _cogl_pipeline_flush_layers_gl_state but there is no need to worry
    * about scaling texture coordinates in this case */
-  tex_handle = _cogl_pipeline_get_layer_texture (pipeline, layer_index);
-  if (tex_handle != COGL_INVALID_HANDLE)
-    _cogl_texture_transform_coords_to_gl (tex_handle, &tx, &ty);
+  texture = _cogl_pipeline_get_layer_texture (pipeline, layer_index);
+  if (texture != NULL)
+    _cogl_texture_transform_coords_to_gl (texture, &tx, &ty);
 
   /* NB: [X,Y,Z,TX,TY...,R,G,B,A,...] */
   t = state->vertices_out + 3 + 2 * state->layer;
@@ -1086,7 +1013,7 @@ cogl_polygon (const CoglTextureVertex *vertices,
 
   for (i = 0; i < n_layers; i++)
     {
-      const char *names[] = {
+      static const char *names[] = {
           "cogl_tex_coord0_in",
           "cogl_tex_coord1_in",
           "cogl_tex_coord2_in",
@@ -1096,8 +1023,13 @@ cogl_polygon (const CoglTextureVertex *vertices,
           "cogl_tex_coord6_in",
           "cogl_tex_coord7_in"
       };
-      char *name = i < 8 ? (char *)names[i] :
-        g_strdup_printf ("cogl_tex_coord%d_in", i);
+      char *allocated_name = NULL;
+      const char *name;
+
+      if (i < 8)
+        name = names[i];
+      else
+        name = allocated_name = g_strdup_printf ("cogl_tex_coord%d_in", i);
 
       attributes[i + 1] = cogl_attribute_new (attribute_buffer,
                                               name,
@@ -1106,6 +1038,8 @@ cogl_polygon (const CoglTextureVertex *vertices,
                                               12 + 8 * i,
                                               2,
                                               COGL_ATTRIBUTE_TYPE_FLOAT);
+
+      g_free (allocated_name);
     }
 
   if (use_color)
@@ -1170,5 +1104,9 @@ cogl_polygon (const CoglTextureVertex *vertices,
 
   if (pipeline != validate_state.original_pipeline)
     cogl_object_unref (pipeline);
-}
 
+  cogl_object_unref (attribute_buffer);
+
+  for (i = 0; i < n_attributes; i++)
+    cogl_object_unref (attributes[i]);
+}
